package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.AuthConstant;
import cn.kdan.pdf.tech.core.constant.MembersConstant;
import cn.kdan.pdf.tech.core.constant.VppMemberConstant;
import cn.kdan.pdf.tech.core.enums.CaptchaActionEnum;
import cn.kdan.pdf.tech.core.enums.ImageCodeTypeEnum;
import cn.kdan.pdf.tech.core.enums.VerifyTypeEnum;
import cn.kdan.pdf.tech.core.mapper.OauthClientDetailsMapper;
import cn.kdan.pdf.tech.core.model.OauthClientDetails;
import cn.kdan.pdf.tech.core.model.OauthClientDetailsExample;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.pojo.oauth2.TokenPOJO;
import cn.kdan.pdf.tech.core.service.AuthService;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import cn.kdan.pdf.tech.core.utils.EmailUtils;
import cn.kdan.pdf.tech.core.utils.TokenUtils;
import constant.CommonConstant;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.http.HttpHeaders;
import org.apache.http.entity.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.common.OAuth2AccessToken;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import pojo.CustomUserDetails;
import utils.*;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.security.Principal;
import java.util.*;

/**
 * 认证实现类
 *
 * @author tangxiangan
 */
@Service
@Slf4j
public class AuthServiceImpl implements AuthService {


    @Autowired
    private OauthClientDetailsMapper oauthClientDetailsMapper;

    @Autowired
    private SMSUtils smsUtils;

    @Autowired
    private EmailUtils emailUtils;

    @Autowired
    private RedisUtils<String, String> redisUtils;

    @Value("${security.oauth2.client.client-id:}")
    private String clientId;

    @Value("${security.oauth2.client.client-secret:}")
    private String secretKey;

    @Value("${readerproUrl}")
    private String readerproUrl;
    @Value("${emailImagesUrl}")
    private String emailImagesUrl;

    @Resource
    private VppMemberService VppMemberService;

    @Override
    public OauthClientDetails getById(String clientId) {
        return oauthClientDetailsMapper.selectByPrimaryKey(clientId);
    }

    @Override
    public List<OauthClientDetails> listAll() {
        return oauthClientDetailsMapper.selectByExample(new OauthClientDetailsExample());
    }

    @Override
    public String getAuthorizeCode(String userId, Principal principal) {
        return TokenUtils.getAuthorizeCode(clientId, principal);
    }

    @Override
    public TokenPOJO getToken(String code) {
        OAuth2AccessToken token = TokenUtils.getToken(code, clientId, secretKey);
        return TokenUtils.convert(token);
    }

    @Override
    public TokenPOJO getTokenByUserId(String userId) {
        VppMember members = VppMemberService.getById(userId);
        Set<SimpleGrantedAuthority> authorities = new LinkedHashSet<>();
        String account = "";
        if (!StringUtils.isEmpty(members.getPhone())) {
            account = members.getPhone();
        } else {
            account = members.getEmail();
        }
        CustomUserDetails principal = new CustomUserDetails(members.getId(),
                account,
                members.getDigestPassword(),
                true,
                true,
                true,
                true,
                authorities);
        UsernamePasswordAuthenticationToken authenticationToken = new UsernamePasswordAuthenticationToken(principal, null, Collections.emptySet());
        OauthClientDetails clientDetails = this.getUsedClient();
        if (clientDetails == null) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_PLEASE_ADD_AUTH_CONFIG);
        }
        //应用内部登录的授权码
        String authCode = TokenUtils.getAuthorizeCode(clientDetails.getClientId(), authenticationToken);
        OAuth2AccessToken token = TokenUtils.getToken(authCode, clientId, secretKey);
        return TokenUtils.convert(token);
    }

    private OauthClientDetails getUsedClient() {

        OauthClientDetailsExample example = new OauthClientDetailsExample();
        example
                .createCriteria()
                .andStateEqualTo(ValidStatusEnum.VALID.value());

        List<OauthClientDetails> oauthClientDetailsList = oauthClientDetailsMapper.selectByExample(example);

        if (CollectionUtils.isEmpty(oauthClientDetailsList)) {
            return null;
        } else {
            return oauthClientDetailsList.get(0);
        }
    }


    @Override
    public void generateImageCode(HttpServletResponse response, HttpSession session, String type, int width, int height, int codeCount, int lineCount) throws IOException {
        ImageCodeTypeEnum imageCodeType;
        //设置相应类型,告诉浏览器输出的内容为图片
        response.setContentType(ContentType.IMAGE_PNG.getMimeType());
        //设置响应头信息，告诉浏览器不要缓存此内容
        response.setHeader(HttpHeaders.PRAGMA, CommonConstant.STRING_NO_CACHE);
        if (StringUtils.isEmpty(type)) {
            imageCodeType = ImageCodeTypeEnum.LOGIN;
        } else {
            imageCodeType = ImageCodeTypeEnum.getEnumByValue(type);
        }
        Map<String, Object> map = ImageCodeUtils.generate(width, height, codeCount, lineCount);
        BufferedImage image = (BufferedImage) map.get(CommonConstant.STRING_IMAGE);
        redisUtils.hset(AuthConstant.SESSION_ + session.getId(), imageCodeType.value(), map.get(CommonConstant.STRING_CODE).toString().toLowerCase());
        response.setHeader(CommonConstant.AUTH_TOKEN, session.getId());
        ServletOutputStream os = response.getOutputStream();
        ImageIO.write(image, CommonConstant.STRING_PNG, os);
        os.close();
    }

    @Override
    public boolean isImageCodeValid(String key, ImageCodeTypeEnum type, String code) {
        String redisCode = redisUtils.hget(AuthConstant.SESSION_ + key, type.value().toLowerCase());
        redisUtils.hdel(AuthConstant.SESSION_ + key, type.value());
        return !StringUtils.isEmpty(code) && code.equalsIgnoreCase(redisCode);
    }

    @Override
    public boolean getVerifyCode(CaptchaActionEnum action, VerifyTypeEnum type, String receiver) {
        boolean flag = false;
        if (redisUtils.hexists(AuthConstant.VERIFY_CODE_TIME_OUT + action.value(), receiver)) {
            throw new BackendRuntimeException(AuthConstant.VERIFY_CODE_SEND_TOO_QUICKLY);
        }
        if (ObjectUtils.isNotEmpty(VppMemberService.getByAccount(receiver))) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_EMAIL_EXIST);
        }

        String code = CommonUtils.generateVerifyCode();
        switch (type) {
            //发送手机验证码
            case PHONE:
                //忘记密码时手机验证 发送手机短信
                if (smsUtils.sendSMS(Collections.singletonList(receiver))) {
                    flag = true;
                }
                break;
            //发送邮件
            case EMAIL:
                //创建账号
                String text = "";
                if(action.value().equals(CaptchaActionEnum.USER_REGISTER.value())){
                    text = String.format(action.text(), receiver, code);
                    // 替换图片动态地址和取消订阅链接
                    text = text.replace("emailImagesUrl", emailImagesUrl).replace("cancelEmailUrl", readerproUrl + "/unsubscribe?email=" + receiver);
                }
                if (emailUtils.sendMail(action.subject(), text, Collections.singletonList(receiver))) {
                    flag = true;
                }
                break;
            default:
                break;
        }
        //存入缓存
        if (flag) {
            redisUtils.hset(AuthConstant.VERIFY_CODE_KEY + action.value(), receiver, code, AuthConstant.VERIFY_CODE_KEY_EXPIRE_TIME);
            //再次发送计时
            redisUtils.hset(AuthConstant.VERIFY_CODE_TIME_OUT + action.value(), receiver, "1", AuthConstant.VERIFY_CODE_KEY_RESEND_TIME);
        }
        return flag;
    }


    @Override
    public boolean checkExist(String key, String account, String code) {
        if (!isImageCodeValid(key, ImageCodeTypeEnum.FORGET_PASSWORD, code)) {
            throw new BackendRuntimeException(AuthConstant.EXCEPTION_MSG_IMAGE_CODE_ERROR);
        }
        if (ObjectUtils.isEmpty(VppMemberService.getByAccount(account))) {
            throw new BackendRuntimeException(MembersConstant.EXCEPTION_MSG_USER_NOT_FOUND);
        }
        return Boolean.TRUE;
    }

    @Override
    public boolean checkVerifyCode(String action, String receiver, String code) {
        String captchaCode = redisUtils.hget(AuthConstant.VERIFY_CODE_KEY + action, receiver);
        return StringUtils.isNotEmpty(code) && code.equals(captchaCode);
    }

    @Override
    public void checkVerifyCodeByAction(CaptchaActionEnum action, String receiver, String code) {
        if (!checkVerifyCode(action.value(), receiver, code)) {
            throw new BackendRuntimeException(VppMemberConstant.EXCEPTION_MSG_FALSE_CODE);
        } else {
            // 验证成功删除验证码缓存
            redisUtils.hdel(AuthConstant.VERIFY_CODE_KEY + action.value(), receiver);
        }
    }

}
