package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.constant.SSOConstant;
import cn.kdan.pdf.tech.core.constant.VppTeamConstant;
import cn.kdan.pdf.tech.core.enums.RoleEnum;
import cn.kdan.pdf.tech.core.enums.ScpoeEnum;
import cn.kdan.pdf.tech.core.mapper.DirectoryMapper;
import cn.kdan.pdf.tech.core.model.*;
import cn.kdan.pdf.tech.core.pojo.MSGroup;
import cn.kdan.pdf.tech.core.pojo.dto.AddSyncDTO;
import cn.kdan.pdf.tech.core.pojo.dto.CreateTeamDTO;
import cn.kdan.pdf.tech.core.pojo.dto.DirectoryCreateDTO;
import cn.kdan.pdf.tech.core.pojo.dto.DirectoryUpdateDTO;
import cn.kdan.pdf.tech.core.pojo.vo.DirectoryVO;
import cn.kdan.pdf.tech.core.pojo.vo.VppTeamMemberVO;
import cn.kdan.pdf.tech.core.service.*;
import com.azure.identity.ClientSecretCredential;
import com.azure.identity.ClientSecretCredentialBuilder;
import com.microsoft.graph.models.Group;
import com.microsoft.graph.models.GroupCollectionResponse;
import com.microsoft.graph.models.User;
import com.microsoft.graph.models.UserCollectionResponse;
import com.microsoft.graph.serviceclient.GraphServiceClient;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import utils.CommonUtils;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
@Transactional
public class DirectoryServiceImpl implements DirectoryService {
    @Resource
    private DirectoryMapper directoryMapper;
    @Resource
    private VppMemberService vppMemberService;
    @Resource
    private DomainService domainService;
    @Resource
    private ScheduleSettingService scheduleSettingService;
    @Resource
    private DirectorySyncService directorySyncService;
    @Resource
    private VppTeamService teamService;
    @Resource
    private VppRTeamMemberRoleService vppRTeamMemberRoleService;
    @Override
    public List<DirectoryVO> list() {
        //获取当前登录用户公司id
        VppMember member = vppMemberService.getCurrentUser();
        //根据公司id查询目录列表
        List<Directory> directories = listByCompanyId(member.getCompanyId());
        List<DirectoryVO> directoryVOS = new ArrayList<>();
        for (Directory directory : directories) {
            DirectoryVO directoryVO = new DirectoryVO();
            BeanUtils.copyProperties(directory, directoryVO);
            if (!StringUtils.isEmpty(directory.getMsGroupIds())){
                directoryVO.setMsGroupIds(directory.getMsGroupIds());
            }
            //根据目录id查询目录下的域名列表
            List<Domain> domains = domainService.listByDirectoryId(directory.getDirectoryId());
            directoryVO.setDomains(domains);

            //遍历目录列表填充最新同步记录
            List<DirectorySync> syncs = directorySyncService.listByDirectoryId(directory.getDirectoryId());
            directoryVO.setSync(CollectionUtils.isEmpty(syncs) ? null : syncs.get(0));

            //遍历目录列表填充同步配置
            List<ScheduleSetting> scheduleSettings = scheduleSettingService.listByDirectoryId(directory.getDirectoryId());
            directoryVO.setScheduleSetting(CollectionUtils.isEmpty(scheduleSettings) ? null : scheduleSettings.get(0));

            directoryVOS.add(directoryVO);
        }
        return directoryVOS;
    }

    @Override
    public Directory create(DirectoryCreateDTO directoryVO) {
        checkExist(directoryVO.getName(),null);
        Directory directory = new Directory();
        //获取当前登录用户公司id
        VppMember member = vppMemberService.getCurrentUser();
        directory.setCompanyId(member.getCompanyId());
        directory.setCreateTime(new Date());
        directory.setUpdateTime(new Date());
        directory.setValidFlag(ValidStatusEnum.VALID.value());
        directory.setName(directoryVO.getName());
        directory.setDirectoryId(CommonUtils.generateId());
        directoryMapper.insert(directory);
        return directory;
    }

    @Override
    public void delete(String directoryId) {
        Directory directory = getById(directoryId);
        domainService.unbindByDirectoryId(directoryId);
        // 同步设置删掉 定时任务删掉
        scheduleSettingService.deleteByDirectoryId(directoryId);
        //删除团队 解绑所有序列码和设备
        if (!StringUtils.isEmpty(directory.getTeamId())){
            teamService.deleteDirectoryTeam(directory.getTeamId());
        }
        // 删除所有团队成员
        vppMemberService.deleteDirectoryMember(directoryId);
        // 删除目录
        directoryMapper.deleteByPrimaryKey(directoryId);
    }

    @Override
    public void update(DirectoryUpdateDTO directoryUpdateDTO) {
        checkExist(directoryUpdateDTO.getName(), directoryUpdateDTO.getDirectoryId());
        Directory directory = new Directory();
        directory.setDirectoryId(directoryUpdateDTO.getDirectoryId());
        directory.setName(directoryUpdateDTO.getName());
        directory.setUpdateTime(new Date());
        directoryMapper.updateByPrimaryKeySelective(directory);
    }


    /**
     * 根据id获取目录
     * @param directoryId  directoryId
     * @return Directory
     */
    @Override
    public Directory getById(String directoryId) {
       Directory directory =  directoryMapper.selectByPrimaryKey(directoryId);
       if(ObjectUtils.isEmpty(directory)){
           throw new BackendRuntimeException(SSOConstant.DIRECTORY_NOT_EXIST);
       }
       return directory;
    }

    @Override
    public void updateSelective(Directory directory) {
        directoryMapper.updateByPrimaryKeySelective(directory);
    }

    private List<Directory> listByCompanyId(String companyId) {
        DirectoryExample example = new DirectoryExample();
        example.createCriteria().andCompanyIdEqualTo(companyId).andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        return directoryMapper.selectByExample(example);
    }
    private void checkExist(String name,String directoryId) {
        DirectoryExample example = new DirectoryExample();
        DirectoryExample.Criteria criteria = example.createCriteria().andNameEqualTo(name).andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        if(!StringUtils.isEmpty(directoryId)){
            criteria.andDirectoryIdNotEqualTo(directoryId);
        }
        List<Directory> list = directoryMapper.selectByExample(example);
        if (!CollectionUtils.isEmpty(list)) {
            throw new BackendRuntimeException(SSOConstant.DIRECTORY_EXIST);
        }
    }

}
