package cn.kdan.pdf.tech.core.service.impl;

import cn.kdan.pdf.tech.core.enums.DomainUsedEnum;
import cn.kdan.pdf.tech.core.enums.DomainVerifiedEnum;
import cn.kdan.pdf.tech.core.enums.DomainVerifyTypeEnum;
import cn.kdan.pdf.tech.core.mapper.DomainMapper;
import cn.kdan.pdf.tech.core.model.Domain;
import cn.kdan.pdf.tech.core.model.DomainExample;
import cn.kdan.pdf.tech.core.model.VppMember;
import cn.kdan.pdf.tech.core.service.DomainService;
import cn.kdan.pdf.tech.core.service.VppMemberService;
import enums.ValidStatusEnum;
import exception.BackendRuntimeException;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClientBuilder;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.xbill.DNS.Lookup;
import org.xbill.DNS.Record;
import org.xbill.DNS.TXTRecord;
import org.xbill.DNS.Type;
import utils.CommonUtils;

import javax.annotation.Resource;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class DomainServiceImpl implements DomainService {
    @Resource
    private DomainMapper domainMapper;
    @Resource
    private VppMemberService vppMemberService;

    @Override
    public List<Domain> listByDirectoryId(String directoryId) {
        DomainExample domainExample = new DomainExample();
        domainExample.createCriteria().andDirectoryIdEqualTo(directoryId).andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        List<Domain> list = domainMapper.selectByExample(domainExample);
        return CollectionUtils.isEmpty(list) ? new ArrayList<>() : list;
    }

    @Override
    public List<Domain> list(String verified, String used) {
        VppMember member = vppMemberService.getCurrentUser();
        DomainExample domainExample = new DomainExample();
        DomainExample.Criteria criteria = domainExample.createCriteria().andCompanyIdEqualTo(member.getCompanyId())
                .andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        if (!StringUtils.isEmpty(verified)) {
            criteria.andVerifiedEqualTo(verified);
        }
        if (!StringUtils.isEmpty(used)) {
            criteria.andUsedEqualTo(verified);
        }
        List<Domain> list = domainMapper.selectByExample(domainExample);
        return CollectionUtils.isEmpty(list) ? new ArrayList<>() : list;
    }

    @Override
    public void delete(String domainId) {
        domainMapper.deleteByPrimaryKey(domainId);
    }

    @Override
    public void unbindByDirectoryId(String directoryId) {
        DomainExample example = new DomainExample();
        example.createCriteria().andDirectoryIdEqualTo(directoryId);
        Domain domain = new Domain();
        domain.setUsed(DomainUsedEnum.AVAILABLE.code());
        domain.setUpdateTime(new Date());
        domain.setDirectoryId("");
        domainMapper.updateByExampleSelective(domain,example);
    }

    @Override
    public void create(String verifyType, String domain) {
        //检查域名是否重复
        checkExist(domain);
        //创建新域名
        Domain domains = new Domain();
        domains.setDomainId(CommonUtils.generateId());
        domains.setDomain(domain);
        domains.setVerifyType(verifyType);
        domains.setCompanyId(vppMemberService.getCurrentUser().getCompanyId());
        domains.setStatus("0");
        domains.setUsed(DomainUsedEnum.AVAILABLE.code());
        domains.setValidFlag(ValidStatusEnum.VALID.value());
        domains.setVerified(DomainVerifiedEnum.UNVERIFIED.code());
        domains.setCreateTime(new Date());
        domains.setUpdateTime(new Date());
        domainMapper.insert(domains);
    }

    @Override
    public void verify(String domainId,String type) {
        Domain domain = getById(domainId);
        domain.setVerifyType(type);
        domainMapper.updateByPrimaryKey(domain);
        //DNS处理方式
        if (DomainVerifyTypeEnum.DNS.code().equals(type)) {
            String txt = "PDF-Tech-Editor" + "-domain-verification=" + domain.getCompanyId();
            checkDns(domain.getDomain(), txt);
        }else{
            checkHttp(domain.getDomain(), "PDF-Tech-Editor-domain-verification.txt", "PDF-Tech-Editor-domain-verification=" + domain.getCompanyId());
        }
        domain.setVerifyType(type);
        domain.setVerified(DomainVerifiedEnum.VERIFIED.code());
        domainMapper.updateByPrimaryKey(domain);
    }

    @Override
    public void bindDomain(String domainId, String directoryId) {
        Domain domain = getById(domainId);
        domain.setDirectoryId(directoryId);
        domain.setUpdateTime(new Date());
        domain.setUsed(DomainUsedEnum.USED.code());
        domainMapper.updateByPrimaryKeySelective(domain);
    }

    @Override
    public void unBind(String domainId) {
        Domain domain = getById(domainId);
        domain.setDirectoryId("");
        domain.setUpdateTime(new Date());
        domain.setUsed(DomainUsedEnum.AVAILABLE.code());
        domainMapper.updateByPrimaryKeySelective(domain);
    }

    private void checkExist(String domain) {
        DomainExample domainExample = new DomainExample();
        domainExample.createCriteria().andDomainEqualTo(domain).andValidFlagEqualTo(ValidStatusEnum.VALID.value());
        List<Domain> list = domainMapper.selectByExample(domainExample);
        if (!CollectionUtils.isEmpty(list)) {
            throw new BackendRuntimeException("Domain name already exists");
        }
    }

    public Domain getById(String domainId) {
        Domain domain = domainMapper.selectByPrimaryKey(domainId);
        if (ObjectUtils.isEmpty(domain)) {
            throw new BackendRuntimeException("Domain does not exist");
        }
        return domain;
    }

    /**
     * 验证域名
     * @param domain    域名
     * @param verifyCode 验证code
     */
    private void checkDns(String domain, String verifyCode){
        boolean found = false;
        try {
            Lookup lookup = new Lookup(domain, Type.TXT);
            Record[] records = lookup.run();
            if (records != null) {
                for (Record record : records) {
                    if (record instanceof TXTRecord) {
                        TXTRecord txtRecord = (TXTRecord) record;
                        String txtValue = txtRecord.rdataToString().replace("\"", ""); // 去掉引号
                        if (txtValue.equals(verifyCode)) {
                            found = true;
                            break;
                        }
                    }
                }
            } else {
                throw new BackendRuntimeException("No domain name resolution record found.");
            }
        } catch (Exception e) {
            throw new BackendRuntimeException("No domain name resolution record found.");
        }
        if (!found) {
            throw new BackendRuntimeException("No domain name resolution record found.");
        }
    }

    /**
     * 验证域名
     * @param domain    域名
     * @param fileName 文件名
     * @param fileContent 文件内容
     */
    private void checkHttp(String domain, String fileName, String fileContent){
        String url = "http://" + domain + "/" + fileName;
        try {
            HttpClient client = HttpClientBuilder.create().build();
            HttpGet request = new HttpGet(url);

            HttpResponse response = client.execute(request);

            BufferedReader reader = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));

            String line;
            StringBuilder result = new StringBuilder();
            while ((line = reader.readLine()) != null) {
                result.append(line);
            }
            reader.close();

            // 在这里对读取到的文件内容进行验证
            String content = result.toString();
            if (!content.contains(fileContent)) {
                throw new BackendRuntimeException("File does not exist.");
            }
        } catch (IOException e) {
            throw new BackendRuntimeException("File does not exist.");
        }
    }
}
