package cn.kdan.pdf.tech.core.utils;

import cn.kdan.pdf.tech.core.enums.ScheduleSettingTypeEnum;
import cn.kdan.pdf.tech.core.model.ScheduleSetting;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;
import org.springframework.stereotype.Component;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import static org.quartz.CronScheduleBuilder.cronSchedule;
import static org.quartz.JobBuilder.newJob;
import static org.quartz.TriggerBuilder.newTrigger;

@Slf4j
@Component
public class QuartzUtils {

    private static Scheduler scheduler;

    static {
        try {
            // 初始化调度器
            scheduler = StdSchedulerFactory.getDefaultScheduler();
            scheduler.start();
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
    }


    public static String generateCronExpression(String timeString) {
        SimpleDateFormat sdfInput = new SimpleDateFormat("HH:mm");
        Date time = null;
        try {
            time = sdfInput.parse(timeString);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }
        SimpleDateFormat sdfOutput = new SimpleDateFormat("ss mm HH * * ?");
        String cronExpression = sdfOutput.format(time);

        return cronExpression;
    }

    public static void scheduleJob(String jobName, String jobGroupName, Class<? extends Job> jobClass,
                                   String triggerName, String triggerGroupName, String cronExpression, String directoryId) {
        try {
            // 创建定时任务
            JobDetail job = newJob(jobClass)
                    .withIdentity(jobName, jobGroupName)
                    .build();

            // 创建触发器
            Trigger trigger = newTrigger()
                    .withIdentity(triggerName, triggerGroupName)
                    .withSchedule(cronSchedule(cronExpression))
                    .build();
            job.getJobDataMap().put("directoryId", directoryId);
            // 关联定时任务和触发器
            scheduler.scheduleJob(job, trigger);
        } catch (Exception e) {
            log.error("定时任务初始化失败", e);
        }
    }

    public static void rescheduleJob(String triggerName, String triggerGroupName, String cronExpression, ScheduleSetting scheduleSetting) {
        try {
            if (scheduleSetting.getType().equals(ScheduleSettingTypeEnum.PER_HOUR.value())) {
                cronExpression = "0 0 * * * ?";
            } else if (scheduleSetting.getType().equals(ScheduleSettingTypeEnum.SPECIFIED_TIME_EVERY_DAY.value())) {
                cronExpression = generateCronExpression(scheduleSetting.getTime());
            }
            // 获取触发器键
            TriggerKey triggerKey = new TriggerKey(triggerName, triggerGroupName);

            // 创建新的触发器
            Trigger newTrigger = newTrigger()
                    .withIdentity(triggerKey)
                    .withSchedule(cronSchedule(cronExpression))
                    .build();

            // 重新调度触发器
            scheduler.rescheduleJob(triggerKey, newTrigger);
        } catch (Exception e) {
            log.error("定时任务重新调度失败", e);
        }
    }

    public static void shutdownScheduler() throws SchedulerException {
        scheduler.shutdown();
    }

    public static void deleteJob(String jobName, String groupName) {
        try {
            // 创建JobKey
            JobKey jobKey = new JobKey(jobName, groupName);

            // 删除任务
            scheduler.deleteJob(jobKey);
        } catch (Exception e) {
            log.error("定时任务删除失败", e);
        }
    }

}
